@@ -6,6 +6,8 @@ from graphql.error import format_error as format_graphql_error
 from starlette.requests import Request
 from starlette.types import Receive, Scope, Send
 from starlette.websockets import WebSocket, WebSocketState
+from strawberry.http import GraphQLHTTPResponse, process_result
+from strawberry.schema import ExecutionResult
 
 from ..schema import BaseSchema
 from ..utils.debug import pretty_print_graphql_operation
@@ -26,7 +28,6 @@ class GraphQL:
     def __init__(
         self,
         schema: BaseSchema,
-        root_value: typing.Any = None,
         graphiql: bool = True,
         keep_alive: bool = False,
         keep_alive_interval: float = 1,
@@ -34,7 +35,6 @@ class GraphQL:
     ) -> None:
         self.schema = schema
         self.graphiql = graphiql
-        self.root_value = root_value
         self.keep_alive = keep_alive
         self.keep_alive_interval = keep_alive_interval
         self._keep_alive_task = None
@@ -48,6 +48,12 @@ class GraphQL:
         else:
             raise ValueError("Unknown scope type: %r" % (scope["type"],))
 
+    async def get_root_value(self, request: Request) -> typing.Optional[typing.Any]:
+        return None
+
+    async def get_context(self, request: Request) -> typing.Optional[typing.Any]:
+        return {"request": request}
+
     async def handle_keep_alive(self, websocket):
         if websocket.application_state == WebSocketState.DISCONNECTED:
             return
@@ -145,17 +151,34 @@ class GraphQL:
 
     async def handle_http(self, scope: Scope, receive: Receive, send: Send):
         request = Request(scope=scope, receive=receive)
-        response = await get_http_response(request, self.execute, self.graphiql)
+        root_value = await self.get_root_value(request)
+        context = await self.get_context(request)
+
+        response = await get_http_response(
+            request=request,
+            execute=self.execute,
+            process_result=self.process_result,
+            graphiql=self.graphiql,
+            root_value=root_value,
+            context=context,
+        )
 
         await response(scope, receive, send)
 
-    async def execute(self, query, variables=None, context=None, operation_name=None):
+    async def process_result(
+        self, request: Request, result: ExecutionResult
+    ) -> GraphQLHTTPResponse:
+        return process_result(result)
+
+    async def execute(
+        self, query, variables=None, context=None, operation_name=None, root_value=None
+    ):
         if self.debug:
             pretty_print_graphql_operation(operation_name, query, variables)
 
         return await self.schema.execute(
             query,
-            root_value=self.root_value,
+            root_value=root_value,
             variable_values=variables,
             operation_name=operation_name,
             context_value=context,
